/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#ifdef  __VCN_ENABLE

#include "Vnc.h"
#include "Root_Interface.h"
#include "Equipement.h"
#include "Modules.h"
#include "Debug.h"

static void viewer_save_screenshot(GtkWidget *vnc, const char *file)
{
      GdkPixbuf *pix = vnc_display_get_pixbuf(VNC_DISPLAY(vnc));
      gdk_pixbuf_save(pix, file, "png", NULL,
                  "tEXt::Generator App", PACKAGE, NULL);
      gdk_pixbuf_unref(pix);
}

static void vnc_screenshot(GtkWidget *menu G_GNUC_UNUSED, GtkWidget *vnc)
{
      GtkWidget *dialog;

      dialog = gtk_file_chooser_dialog_new ("Save screenshot",
                                    NULL,
                                    GTK_FILE_CHOOSER_ACTION_SAVE,
                                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                    GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                    NULL);
      gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
      if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
            char *filename;

            filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
            viewer_save_screenshot(vnc, filename);
            g_free (filename);
      }
      gtk_widget_destroy (dialog);
}


static void vnc_grab(GtkWidget *vnc, GtkWidget *window)
{

}

static void vnc_ungrab(GtkWidget *vnc, GtkWidget *window)
{

	gtk_widget_grab_focus (window_root);
}

static void vnc_connected(GtkWidget *vnc G_GNUC_UNUSED)
{

	Equipement_Selectionner->password_failed = TRUE;
	Equipement_Selectionner->password_vnc_ok = FALSE;
}

static void vnc_initialized(GtkWidget *vnc, GtkWidget *window)
{

	Equipement_Selectionner->password_failed = FALSE;
	Equipement_Selectionner->password_vnc_ok = TRUE;	
	gtk_widget_show_all(window);
}

static void vnc_auth_failure(GtkWidget *vnc, const char *msg)
{

	Equipement_Selectionner->password_failed = TRUE;
	Equipement_Selectionner->password_vnc_ok = FALSE;
}

static void vnc_credential(GtkWidget *vncdisplay, GValueArray *credList)
{
	GtkWidget *dialog = NULL;
	int response;
	unsigned int i, prompt = 0;
	const char **data;



	data = g_new0(const char *, credList->n_values);

	for (i = 0 ; i < credList->n_values ; i++) {
		GValue *cred = g_value_array_get_nth(credList, i);
		switch (g_value_get_enum(cred)) {
		case VNC_DISPLAY_CREDENTIAL_USERNAME:
		case VNC_DISPLAY_CREDENTIAL_PASSWORD:
			if  ((Equipement_Selectionner->Password_VNC_Default) && (Reseau_Selectionner->Global_Vnc_Password != NULL))
				data[i] =Reseau_Selectionner->Global_Vnc_Password;
			else
				data[i] = Equipement_Selectionner->password_vnc; 
			printf("Password : [%s]\n",data[i]);
			if (vnc_display_set_credential(VNC_DISPLAY(vncdisplay), g_value_get_enum(cred), data[i])) {
				printf("Failed to set credential type %d\n", g_value_get_enum(cred));
			}
			break;
		case VNC_DISPLAY_CREDENTIAL_CLIENTNAME:
			data[i] = "autoscan-network";
		default:
			break;
		}
	}
	g_free(data);
}

static void vnc_desktop_resize(GtkWidget *vnc, int width, int height)
{

}

static void vnc_disconnected(GtkWidget *vnc G_GNUC_UNUSED,struct Struct_Equipement *Equipement)
{

	gtk_widget_grab_focus (window_root);
	if (Equipement_Selectionner == Equipement){
	
			gtk_window_unfullscreen(GTK_WINDOW(window_root));
			gtk_widget_show(toolbar);
			gtk_widget_show(viewport_Notebook_Info);
			gtk_widget_show(vbox_network);
			gtk_widget_show(hbox_Milieu);
		
	
		if (Equipement_Selectionner->password_failed)
			Initialisation_Vnc();
		else {
		
			Equipement_Selectionner->combo_sauv = 0;
			gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_liste_Modules), Equipement_Selectionner->combo_sauv);
		
		}
	
	
	}
	 
}

static void send_caf1(GtkWidget *menu G_GNUC_UNUSED, GtkWidget *vnc)
{
	guint keys[] = { GDK_Control_L, GDK_Alt_L, GDK_F1 };

	vnc_display_send_keys(VNC_DISPLAY(vnc), keys, sizeof(keys)/sizeof(keys[0]));
}

static void send_caf7(GtkWidget *menu G_GNUC_UNUSED, GtkWidget *vnc)
{
	guint keys[] = { GDK_Control_L, GDK_Alt_L, GDK_F7 };

	vnc_display_send_keys(VNC_DISPLAY(vnc), keys, sizeof(keys)/sizeof(keys[0]));
}

static void send_cad(GtkWidget *menu G_GNUC_UNUSED, GtkWidget *vnc)
{
	guint keys[] = { GDK_Control_L, GDK_Alt_L, GDK_Delete };

	vnc_display_send_keys(VNC_DISPLAY(vnc), keys, sizeof(keys)/sizeof(keys[0]));
}

static void send_cab(GtkWidget *menu G_GNUC_UNUSED, GtkWidget *vnc)
{
	guint keys[] = { GDK_Control_L, GDK_Alt_L, GDK_BackSpace };

	vnc_display_send_keys(VNC_DISPLAY(vnc), keys, sizeof(keys)/sizeof(keys[0]));
}


static void do_fullscreen(GtkWidget *menu, GtkWidget *window)
{
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu))){
		gtk_widget_hide(toolbar);
		gtk_widget_hide(viewport_Notebook_Info);
		gtk_widget_hide(vbox_network);
		gtk_widget_hide(hbox_Milieu);
	
		gtk_window_fullscreen(GTK_WINDOW(window));
	}
	else {
		gtk_window_unfullscreen(GTK_WINDOW(window));
		gtk_widget_show(toolbar);
		gtk_widget_show(viewport_Notebook_Info);
		gtk_widget_show(vbox_network);
		gtk_widget_show(hbox_Milieu);
	
	}
}

static void do_scaling(GtkWidget *menu, GtkWidget *vncdisplay)
{
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu)))
		vnc_display_set_scaling(VNC_DISPLAY(vncdisplay), TRUE);
	else
		vnc_display_set_scaling(VNC_DISPLAY(vncdisplay), FALSE);
}

static void delete_event(GtkWidget *menu, GtkWidget *vncdisplay)
{

}

void Vnc_Console (gchar *password) {
	struct Struct_Equipement *Equipement;
	char port[1024], hostname[1024];
	char *display = NULL;
	GtkWidget *vnc_display = NULL;
	GtkWidget *layout_vnc = NULL;
	GtkWidget *viewport_Vnc = NULL;
	GtkWidget *menubar = NULL;
	GtkWidget *sendkey = NULL;
	GtkWidget *affichage = NULL;
	GtkWidget *submenu = NULL;
	GtkWidget *caf1 = NULL;
	GtkWidget *caf7 = NULL;
	GtkWidget *cad = NULL;
	GtkWidget *cab = NULL;
	GtkWidget *shot = NULL;
	GtkWidget *fullscreen_vnc = NULL;
	GtkWidget *scaling = NULL;
	GtkWidget *hbox_Vnc = NULL,*fixed_Vnc_Gauche = NULL,*vbox_Vnc_Milieu = NULL,*fixed_Vnc_Milieu_Bas = NULL,*fixed_Vnc_Droite = NULL;
  	

	Equipement = Equipement_Selectionner;
	

	scrolledwindow_Vnc = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_Vnc);
	gtk_container_add (GTK_CONTAINER (viewport_Modules), scrolledwindow_Vnc);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow_Vnc),GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);


	viewport_Vnc = gtk_viewport_new (NULL, NULL);
  	gtk_widget_show (viewport_Vnc);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_Vnc), viewport_Vnc);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport_Vnc), GTK_SHADOW_NONE);
		

	vnc_display = vnc_display_new();

	layout_vnc = gtk_vbox_new(FALSE, 3);

	menubar = gtk_menu_bar_new();

	sendkey = gtk_menu_item_new_with_mnemonic("_Send Key");
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), sendkey);
	submenu = gtk_menu_new();
	caf1 = gtk_menu_item_new_with_mnemonic("Ctrl+Alt+F_1");
	caf7 = gtk_menu_item_new_with_mnemonic("Ctrl+Alt+F_7");
	cad = gtk_menu_item_new_with_mnemonic("Ctrl+Alt+_Del");
	cab = gtk_menu_item_new_with_mnemonic("Ctrl+Alt+_Backspace");
	shot = gtk_menu_item_new_with_mnemonic("Screenshot");
	gtk_menu_append(GTK_MENU(submenu), caf1);
	gtk_menu_append(GTK_MENU(submenu), caf7);
	gtk_menu_append(GTK_MENU(submenu), cad);
	gtk_menu_append(GTK_MENU(submenu), cab);
	gtk_menu_append(GTK_MENU(submenu), shot);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(sendkey), submenu);

	affichage = gtk_menu_item_new_with_mnemonic("_View");
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), affichage);
	submenu = gtk_menu_new();
	fullscreen_vnc = gtk_check_menu_item_new_with_mnemonic("_Full Screen");
	scaling = gtk_check_menu_item_new_with_mnemonic("Scaled display");
	gtk_menu_append(GTK_MENU(submenu), fullscreen_vnc);
	gtk_menu_append(GTK_MENU(submenu), scaling);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(affichage), submenu);
		

	gtk_container_add(GTK_CONTAINER(viewport_Vnc), layout_vnc);
	gtk_box_pack_start (GTK_BOX (layout_vnc), menubar, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(layout_vnc), vnc_display);


	if (password != NULL) { 
		free_gchar_if_no_null (Equipement->password_vnc);
		Equipement->password_vnc = g_strdup (password);
	}

	ERROR_if (Equipement_Selectionner->Vnc_Port == NULL);

	vnc_display_open_host(VNC_DISPLAY(vnc_display), Equipement->Adresse_Ip_Char,Equipement_Selectionner->Vnc_Port);

	if (!Equipement_Selectionner->View_Only_VNC){
		vnc_display_set_keyboard_grab(VNC_DISPLAY(vnc_display), TRUE);
		vnc_display_set_pointer_grab(VNC_DISPLAY(vnc_display), TRUE);
	}
	else
		vnc_display_set_read_only(VNC_DISPLAY(vnc_display), TRUE);
	gtk_widget_grab_focus (VNC_DISPLAY(vnc_display));


	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-connected",  GTK_SIGNAL_FUNC(vnc_connected), NULL);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-initialized",  GTK_SIGNAL_FUNC(vnc_initialized), viewport_Vnc);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-disconnected",   GTK_SIGNAL_FUNC(vnc_disconnected), Equipement);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-auth-failure",	   GTK_SIGNAL_FUNC(vnc_auth_failure), NULL);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-auth-credential",   GTK_SIGNAL_FUNC(vnc_credential), NULL);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-desktop-resize",  GTK_SIGNAL_FUNC(vnc_desktop_resize), NULL);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-pointer-grab",   GTK_SIGNAL_FUNC(vnc_grab), viewport_Vnc);
	gtk_signal_connect(GTK_OBJECT(vnc_display), "vnc-pointer-ungrab",  GTK_SIGNAL_FUNC(vnc_ungrab), viewport_Vnc);
	gtk_signal_connect(GTK_OBJECT(caf1), "activate",   GTK_SIGNAL_FUNC(send_caf1), vnc_display);
	gtk_signal_connect(GTK_OBJECT(caf7), "activate",  GTK_SIGNAL_FUNC(send_caf7), vnc_display);
	gtk_signal_connect(GTK_OBJECT(cad), "activate",  GTK_SIGNAL_FUNC(send_cad), vnc_display);
	gtk_signal_connect(GTK_OBJECT(cab), "activate",   GTK_SIGNAL_FUNC(send_cab), vnc_display);
	gtk_signal_connect(GTK_OBJECT(shot), "activate",   GTK_SIGNAL_FUNC(vnc_screenshot), vnc_display);
	gtk_signal_connect(GTK_OBJECT(viewport_Vnc), "delete-event",  GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(fullscreen_vnc), "toggled",  GTK_SIGNAL_FUNC(do_fullscreen), window_root);
	gtk_signal_connect(GTK_OBJECT(scaling), "toggled",  GTK_SIGNAL_FUNC(do_scaling), vnc_display);
}
#endif
